<?php

namespace App\Connector\Components;

class MarketplaceProduct extends \App\Core\Components\Base
{
    /**
     * Create marketplace specific product table
     * @param $userId
     * @param $marketplace
     * @param array $product
     * @return bool
     */
    public function createMarketplaceProduct($userId, $marketplace, $product = [])
    {
        $this->di->getObjectManager()->get('\App\Connector\Models\MarketplaceProduct')
            ->createMarketplaceTables($userId, $marketplace, $product);
        return true;
    }

    private function getQueryForPStatus($marketplace)
    {
        $aggregate = [];

        $aggregate[] = [
            '$match' => [
                'user_id' => $marketplace['user_id'],
                // 'marketplace.mercado_cbt' => ['$exists' => true],
            ],
        ];

        $aggregate[] = [
            '$match' => [
                '$or' => [
                    [
                        '$and' => [
                            ['type' => 'variation'],
                            ["visibility" => 'Catalog and Search'],
                        ],
                    ],
                    [
                        '$and' => [
                            ['type' => 'simple'],
                            ["visibility" => 'Catalog and Search'],
                        ],
                    ],
                ],
            ],
        ];

        $aggregate[] = ['$unwind' => '$marketplace.mercado_cbt'];

        $aggregate[] = [
            '$group' => [
                '_id' => '$marketplace.mercado_cbt.status',
                'count' => ['$sum' => 1],
            ],
        ];

        return $aggregate;
    }

    public function getproductStatus($marketplace)
    {
        echo "<pre>";

        // $this->di->getLog()->logContent('$marketplace: ' . json_encode($marketplace, 128), 'info', 'ShubhLog.log');

        $mongo = $this->di->getObjectManager()->create('\App\Core\Models\BaseMongo');
        $collection = $mongo->getCollectionForTable('product_container');

        $totalAggreagte = [];

        $totalAggreagte[] = ['$count' => 'count'];

        $aggregate = [];

        // $aggregate[] = [
        //     '$match' => [
        //         'marketplace.mercado_cbt' => ['$exists' => true],
        //         'marketplace.mercado_cbt.target_product_id' => ['$exists' => true],
        //     ],
        // ];
        // $aggregate[] = ['$count' => 'count'];
        // $aggregateStatus[] = ['$unwind' => '$marketplace.mercado_cbt'];

        $aggregateStatus[] = [
            '$group' => [
                '_id' => '$status',
                'count' => ['$sum' => 1],
            ],
        ];

        $facet = [
            [
                '$match' => [
                    'user_id' => $marketplace['user_id'],
                    '$or' => [
                        [
                            '$and' => [
                                ['type' => 'variation'],
                                ["visibility" => 'Catalog and Search'],
                            ],
                        ],
                        [
                            '$and' => [
                                ['type' => 'simple'],
                                ["visibility" => 'Catalog and Search'],
                            ],
                        ],
                    ],
                ],
            ],
            [
                '$facet' => [
                    'total' => $totalAggreagte,
                    'uploaded' => $aggregate,
                    'status' => $aggregateStatus,
                ],
            ],
        ];

        $data = $collection->aggregate($facet)->toArray();
        $data = json_decode(json_encode($data), true);

        $newData = [];
        foreach ($data[0] as $key => $value) {
            if ($key === 'status') {
                // if ($value['_id'] === null) {
                //     $value['_id'] = 'Not Uploaded';
                // }
                $newData[$key] = $value;
            } elseif (isset($value[0]['count'])) {
                $newData[$key] = $value[0]['count'];
            } else {
                $newData[$key] = 0;
            }
        }
        // die;

        // $newData['status'] = $this->moldUploadStatus($newData);

        return [
            'success' => true,
            'data' => $newData,
        ];
    }

    public function moldUploadStatus($var)
    {
        $totalStatus = 0;
        foreach ($var['status'] as $key => $value) {
            $totalStatus = $totalStatus + $value['count'];
        }

        $var['status'][] = [
            '_id' => 'not uploaded',
            'count' => abs($var['total'] - $totalStatus),
        ];

        return $var['status'];
    }
}
